@@ -1,7 +1,7 @@
 /*
- * Copyright 2004-2007 J. Dahl and L. Vandenberghe.
+ * Copyright 2004-2008 J. Dahl and L. Vandenberghe.
  *
- * This file is part of CVXOPT version 0.9.
+ * This file is part of CVXOPT version 0.9.3.
  *
  * CVXOPT is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,13 +27,13 @@
 
 PyDoc_STRVAR(base__doc__,"Convex optimization package");
 
-PyTypeObject matrix_tp ;
+extern PyTypeObject matrix_tp ;
 matrix * Matrix_New(int, int, int) ;
 matrix * Matrix_NewFromMatrix(matrix *, int) ;
 matrix * Matrix_NewFromSequence(PyObject *, int) ; 
 matrix * Matrix_NewFromArrayStruct(PyObject *, int, int *) ;
 
-PyTypeObject spmatrix_tp ;
+extern PyTypeObject spmatrix_tp ;
 spmatrix * SpMatrix_New(int_t, int_t, int, int ) ;
 spmatrix * SpMatrix_NewFromMatrix(matrix *, int) ;
 spmatrix * SpMatrix_NewFromSpMatrix(spmatrix *, int, int) ; 
@@ -57,7 +57,6 @@ extern int (*sp_syrk[])(char, char, number, void *, number,
 
 const int  E_SIZE[] = { sizeof(int_t), sizeof(double), sizeof(complex) };
 const char TC_CHAR[][2] = {"i","d","z"} ;
-const char PRINTOPT[][15] = {"iformat","dformat","zformat"} ;
 
 PyObject *base_mod;
 
@@ -841,12 +840,7 @@ sparse(PyTypeObject *type, PyObject *args, PyObject *kwds)
     }
     
     for (jk=0; jk<SP_NCOLS(Objx); jk++) 
-      SP_COL(ret)[jk+1] += SP_COL(ret)[jk];
-    
-    /*
-    ret = SpMatrix_NewFromSpMatrix((spmatrix *)Objx, 0, 
-	(id == -1 ? SP_ID(Objx) : id));
-    */
+      SP_COL(ret)[jk+1] += SP_COL(ret)[jk];    
   }
 
   /* x is a list of lists */
@@ -858,6 +852,129 @@ sparse(PyTypeObject *type, PyObject *args, PyObject *kwds)
   return (PyObject *)ret;
 }
 
+static char doc_spdiag[] =
+    "Constructs a square block diagonal sparse matrix.\n\n"
+    "spdiag(diag)\n\n"
+    "ARGUMENTS\n"
+    "diag      a matrix with a single row or column,  or a list of matrices\n"
+    "          and scalars.";
+
+static PyObject *
+spdiag(PyTypeObject *type, PyObject *args, PyObject *kwds)
+{
+  PyObject *diag = NULL, *Dk;
+  static char *kwlist[] = { "diag", NULL};
+
+  if (!PyArg_ParseTupleAndKeywords(args, kwds, "O", kwlist, &diag))
+    return NULL;
+  
+  if ((!PyList_Check(diag) && !Matrix_Check(diag)) ||
+      (Matrix_Check(diag) && 
+	  (MAT_LGT(diag) != MAX(MAT_NROWS(diag),MAT_NCOLS(diag)))))
+    PY_ERR_TYPE("invalid diag argument");
+  
+  if (Matrix_Check(diag)) {
+    int j, id = MAX(DOUBLE, MAT_ID(diag)), n = MAT_LGT(diag);
+
+    spmatrix *ret = SpMatrix_New(n, n, n, id);
+    if (!ret) return PyErr_NoMemory();
+    SP_COL(ret)[0] = 0;
+    
+    for (j=0; j<n; j++) {
+      SP_COL(ret)[j+1] = j+1;
+      SP_ROW(ret)[j] = j;
+
+      if (MAT_ID(diag) == INT)	
+	SP_VALD(ret)[j] = MAT_BUFI(diag)[j];
+      else if (MAT_ID(diag) == DOUBLE)
+	SP_VALD(ret)[j] = MAT_BUFD(diag)[j];
+      else
+	SP_VALZ(ret)[j] = MAT_BUFZ(diag)[j];
+    }
+    return (PyObject *)ret;
+  }
+
+  int j, k, l, idx, id=DOUBLE, n=0, nnz=0;
+
+  for (k=0; k<PyList_GET_SIZE(diag); k++) {
+    Dk = PyList_GET_ITEM(diag, k);
+    if (!Matrix_Check(Dk) && !SpMatrix_Check(Dk) && !PyNumber_Check(Dk))
+      PY_ERR_TYPE("invalid element in diag");
+    
+    if (PyNumber_Check(Dk)) {
+      int scalarid = (PyComplex_Check(Dk) ? COMPLEX :
+	  (PyFloat_Check(Dk) ? DOUBLE : INT));
+      id = MAX(id, scalarid);
+      nnz += 1;
+      n += 1;
+    } else {
+      if (X_NROWS(Dk) != X_NCOLS(Dk))
+	PY_ERR_TYPE("the elements in diag must be square");
+
+      n   += X_NCOLS(Dk);
+      nnz += (Matrix_Check(Dk) ? X_NROWS(Dk)*X_NROWS(Dk) : SP_NNZ(Dk));
+      id   = MAX(id, X_ID(Dk));
+    }
+  }
+
+  spmatrix *ret = SpMatrix_New(n, n, nnz, id);
+  if (!ret) return PyErr_NoMemory();
+  SP_COL(ret)[0] = 0;
+
+  n = 0, idx = 0;
+  for (k=0; k<PyList_GET_SIZE(diag); k++) {
+    Dk = PyList_GET_ITEM(diag, k);
+  
+    if (PyNumber_Check(Dk)) {
+      SP_COL(ret)[n+1] = SP_COL(ret)[n] + 1;
+      SP_ROW(ret)[idx] = n;
+
+      number val;
+      convert_num[id](&val, Dk, 1, 0);
+      write_num[id](SP_VAL(ret), idx, &val, 0);
+      idx += 1;
+      n   += 1;
+    }
+    else {
+      for (j=0; j<X_NCOLS(Dk); j++) {
+
+	if (Matrix_Check(Dk)) {
+	  
+	  SP_COL(ret)[j+n+1] = SP_COL(ret)[j+n] + X_NROWS(Dk);
+	  for (l=0; l<X_NROWS(Dk); l++) {
+	    SP_ROW(ret)[idx] = n + l;
+	    if (id == DOUBLE) 
+	      SP_VALD(ret)[idx] = (MAT_ID(Dk) == DOUBLE ? 
+		  MAT_BUFD(Dk)[l + j*MAT_NROWS(Dk)] :
+		  MAT_BUFI(Dk)[l + j*MAT_NROWS(Dk)]);
+	    else
+	      SP_VALZ(ret)[idx] = (MAT_ID(Dk) == COMPLEX ?
+		  MAT_BUFZ(Dk)[l + j*MAT_NROWS(Dk)] :
+		  (MAT_ID(Dk) == DOUBLE ? MAT_BUFD(Dk)[l + j*MAT_NROWS(Dk)] :
+		      MAT_BUFI(Dk)[l + j*MAT_NROWS(Dk)]));
+	    
+	    idx++;
+	  }      
+	} else {
+	  
+	  SP_COL(ret)[j+n+1] = SP_COL(ret)[j+n] + SP_COL(Dk)[j+1]-SP_COL(Dk)[j];
+	  for (l=SP_COL(Dk)[j]; l<SP_COL(Dk)[j+1]; l++) {
+	    SP_ROW(ret)[idx] = n + SP_ROW(Dk)[l];
+	    if (id == DOUBLE) 
+	      SP_VALD(ret)[idx] = SP_VALD(Dk)[l];
+	    else
+	      SP_VALZ(ret)[idx] = (SP_ID(Dk) == COMPLEX ?
+		  SP_VALZ(Dk)[l] : SP_VALD(Dk)[l]);	  
+	    idx++;
+	  }
+	}
+      }
+      n += X_NCOLS(Dk);    
+    }
+  }
+  return (PyObject *)ret;
+}
+
 extern PyObject * matrix_exp(matrix *, PyObject *, PyObject *) ;
 extern PyObject * matrix_log(matrix *, PyObject *, PyObject *) ;
 extern PyObject * matrix_sqrt(matrix *, PyObject *, PyObject *) ;
@@ -888,6 +1005,7 @@ static PyMethodDef base_functions[] = {
    "elementwise division between two matrices"},
   {"sparse", (PyCFunction)sparse, METH_VARARGS|METH_KEYWORDS, 
    "convenience function for creating sparse matrices"},
+  {"spdiag", (PyCFunction)spdiag, METH_VARARGS|METH_KEYWORDS, doc_spdiag},
   {NULL}		/* sentinel */
 };
 
@@ -922,24 +1040,6 @@ initbase(void)
   if (PyModule_AddObject(base_mod, "spmatrix", (PyObject *) &spmatrix_tp) < 0)
     return;   
 
-  /* Create the print_options dictionary */
-  PyObject *printopt = PyDict_New();
-  PyObject *dstr = PyString_FromString("5.4e");
-  PyObject *istr = PyString_FromString("5li");
-  PyObject *zstr = PyString_FromString("5.4e");
-  if (PyDict_SetItemString(printopt, "dformat", dstr) ||
-      PyDict_SetItemString(printopt, "iformat", istr) ||
-      PyDict_SetItemString(printopt, "zformat", zstr)) 
-  {
-    Py_XDECREF(printopt); 
-    Py_XDECREF(dstr); Py_XDECREF(istr); Py_XDECREF(zstr);
-    return;
-  }
-  Py_DECREF(dstr); Py_DECREF(istr); Py_DECREF(zstr);
-
-  if (PyModule_AddObject(base_mod, "print_options", printopt) < 0)
-    return;   
-
   One[INT].i = 1; One[DOUBLE].d = 1.0; One[COMPLEX].z = 1.0;
 
   MinusOne[INT].i = -1; MinusOne[DOUBLE].d = -1.0; MinusOne[COMPLEX].z = -1.0;
