@@ -1,7 +1,7 @@
 /*
- * Copyright 2004-2007 J. Dahl and L. Vandenberghe.
+ * Copyright 2004-2008 J. Dahl and L. Vandenberghe.
  *
- * This file is part of CVXOPT version 0.9.
+ * This file is part of CVXOPT version 0.9.3.
  *
  * CVXOPT is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,7 +41,7 @@ typedef struct {
 static const char PY_ARRAY_TC[3] = { 'i', 'f', 'c' }; 
 
 /* prototyping and forward declarations */
-PyObject *base_mod;
+extern PyObject *base_mod;
 extern void (*axpy[])(int *, number *, void *, int *, void *, int *) ;
 extern void (*scal[])(int *, number *, void *, int *) ;
 extern void (*gemm[])(char *, char *, int *, int *, int *, void *, void *, 
@@ -57,17 +57,11 @@ int get_id(void *, int ) ;
 
 extern const int  E_SIZE[];
 extern const char TC_CHAR[][2];
-extern const char PRINTOPT[][15];
 extern number One[3], MinusOne[3], Zero[3];
 
+extern PyObject *base_mod;
 
-PyTypeObject spmatrix_tp ;
-/*
-spmatrix * SpMatrix_New(int_t, int_t, int, int ) ;
-spmatrix * SpMatrix_NewFromSpMatrix(spmatrix *, int, int) ; 
-spmatrix * SpMatrix_NewFromIJV(matrix *, matrix *, matrix *, 
-    int_t, int_t, int, int) ;
-*/
+extern PyTypeObject spmatrix_tp ;
 
 PyTypeObject matrix_tp ;
 matrix * Matrix_NewFromNumber(int , int , int , void *, int ) ;
@@ -307,7 +301,7 @@ matrix * Matrix_NewFromSequence(PyObject *x, int id)
     }    
   }
 
-  if (!len) return Matrix_New(0, 0, (id < 0 ? INT : id));
+  if (!len) return Matrix_New(0, 1, (id < 0 ? INT : id));
 
   matrix *L = Matrix_New(len,1,id);
   if (!L) { Py_DECREF(seq); return (matrix *)PyErr_NoMemory(); }
@@ -461,7 +455,7 @@ matrix_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
   int nrows=0, ncols=0;
   char tc = 0;
 
-  if (!PyArg_ParseTupleAndKeywords(args, kwds, "O|Oc:matrix", kwlist, 
+  if (!PyArg_ParseTupleAndKeywords(args, kwds, "|OOc:matrix", kwlist, 
 	  &Objx, &size, &tc))
     return NULL;
 
@@ -474,6 +468,10 @@ matrix_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
   if (tc && !(VALID_TC_MAT(tc))) PY_ERR_TYPE("tc must be 'i', 'd' or 'z'");  
   int id = (tc ? TC2ID(tc) : -1);
 
+  if (!Objx && size) PY_ERR_TYPE("invalid arguments");
+
+  if (!Objx) return (PyObject *)Matrix_New(0, 0, (id == -1 ? INT : id));
+
   matrix *ret = NULL;
   /* x is a number */
   if (PY_NUMBER(Objx)) 
@@ -528,87 +526,35 @@ matrix_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
 }
 
 static PyObject *
-matrix_str(matrix *self)
-{  
-  PyObject *printopt;
+matrix_str(matrix *self) {
 
-  /* these PyObject variables will all be borrowed references */
-  PyObject *opt, *s, *stmp=NULL;
-
-  if (self->nrows == 0 || self->ncols == 0)
-    return Py_BuildValue("s","");
-  
-  if (!(printopt = PyObject_GetAttrString(base_mod, "print_options")) ||
-      !PyDict_Check(printopt)) {
-    PY_ERR(PyExc_AttributeError, "cannot access 'print_options' dictionary");
-  }
- 
-  if (!(opt = PyDict_GetItemString(printopt, PRINTOPT[self->id]))) {
-    Py_DECREF(printopt);
-    PY_ERR(PyExc_KeyError, "missing print format key in 'base_options'");
-  }
-
-  char tmp[50], format[20] = "%- ", format_im[20] = "%";
-  strncat(format, PyString_AsString(opt), 18);
-  if (MAT_ID(self)==COMPLEX)
-    strncat(format_im, PyString_AsString(opt), 18);
-
-  if (!(s = PyString_FromString(""))) {
-    Py_DECREF(printopt);
-    return PyErr_NoMemory();
-  }
-
-  int i, j;
-  for (i=0; i<self->nrows; i++) {
-
-    for (j=0; j<self->ncols; j++) {
-      PyString_ConcatAndDel(&s, PyString_FromString("  "));
-      switch (MAT_ID(self)) {
-      case DOUBLE: 
-	if ((snprintf(tmp,50,format, MAT_BUFD(self)[j*self->nrows+i])<0) ||
-	    !(stmp = PyString_FromString(tmp)))
-	  goto fail;
-	break;
-      case INT: 
-	if ((snprintf(tmp,50,format,MAT_BUFI(self)[j*self->nrows+i])<0) ||
-	    !(stmp = PyString_FromString(tmp)))
-	  goto fail;
-	break;
-      case COMPLEX:
-	if ((snprintf(tmp,50,format,
-		    creal(MAT_BUFZ(self)[j*self->nrows+i]))<0) ||
-	    !(stmp = PyString_FromString(tmp)))
-	  goto fail;
-	
-	if (snprintf(tmp,50,format_im, 
-		fabs(cimag(MAT_BUFZ(self)[j*self->nrows+i])))<0)
-	  goto fail;
-	
-	PyString_ConcatAndDel(&stmp, cimag(MAT_BUFZ(self)[j*self->nrows+i])>0 ?
-	    PyString_FromString("+j") : PyString_FromString("-j"));
-	PyString_ConcatAndDel(&stmp, PyString_FromString(tmp));       
-	break;
-      }   
-      PyString_ConcatAndDel(&s, stmp);
-    }
-    PyString_ConcatAndDel(&s, PyString_FromString("\n"));
-  }
-  
-  Py_DECREF(printopt);
-  return s;
+  PyObject *str, *ret;
    
- fail:
-  Py_DECREF(printopt); 
-  Py_DECREF(s); return PyErr_NoMemory();
+  if (!(str = PyObject_GetAttrString(base_mod, "matrix_str")))
+    PY_ERR(PyExc_KeyError, "missing 'matrix_str' in 'base_options'");
+
+  if (!PyCallable_Check(str)) PY_ERR_TYPE("'matrix_str' is not callable");
+
+  ret = PyObject_CallFunctionObjArgs(str, (PyObject *)self, NULL);
+  Py_DECREF(str);
+
+  return ret;  
 }
 
 static PyObject *
 matrix_repr(matrix *self) {
 
-  PyObject *s = PyString_FromFormat("<%ldx%ld matrix, tc='%s'>",
-      (long)self->nrows,(long)self->ncols,TC_CHAR[self->id]);
+  PyObject *repr, *ret;
+   
+  if (!(repr = PyObject_GetAttrString(base_mod, "matrix_repr")))
+    PY_ERR(PyExc_KeyError, "missing 'matrix_repr' in 'base_options'");
 
-  return s;
+  if (!PyCallable_Check(repr)) PY_ERR_TYPE("'matrix_repr' is not callable");
+
+  ret = PyObject_CallFunctionObjArgs(repr, (PyObject *)self, NULL);
+  Py_DECREF(repr);
+
+  return ret;
 }
 
 /*
