@@ -1,20 +1,21 @@
 """
 Convex optimization solvers.
 
-conelp:  solves cone programs.
-cp:      solves nonlinear convex problems.
-lp:      solves linear programs.
-gp:      solves geometric programs.
-nlcp:    solves nonlinear convex problems specified via functions.
-qp:      solves quadratic programs.
-sdp:     solves semidefinite programs.
-socp:    solves second-order cone programs.
-options: dictionary with customizable algorithm parameters.
+conelp:   solves linear cone programs.
+coneqp:   solves quadratic cone programs.
+cp:       solves nonlinear convex problem.
+cpl:      solves nonlinear convex problems with linear objectives.
+gp:       solves geometric programs.
+lp:       solves linear programs.
+qp:       solves quadratic programs.
+sdp:      solves semidefinite programs.
+socp:     solves second-order cone programs.
+options:  dictionary with customizable algorithm parameters.
 """
 
-# Copyright 2004-2007 J. Dahl and L. Vandenberghe.
+# Copyright 2004-2008 J. Dahl and L. Vandenberghe.
 # 
-# This file is part of CVXOPT version 0.9.
+# This file is part of CVXOPT version 0.9.3.
 #
 # CVXOPT is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -30,10 +31,9 @@ options: dictionary with customizable algorithm parameters.
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import cvxopt
-from cvxopt.cvxprog import cp, qp, gp, nlcp
-from cvxopt.coneprog import conelp, lp, sdp, socp
-solvecp, solveqp, solvegp, solvelp, solvesdp = cp, qp, gp, lp, sdp
+from cvxopt.cvxprog import cp, cpl, gp 
+from cvxopt.coneprog import conelp, lp, sdp, socp, coneqp, qp
 options = {}
 cvxopt.cvxprog.options = options
 cvxopt.coneprog.options = options
-__all__ = ['conelp', 'cp', 'lp', 'gp', 'nlcp', 'qp', 'sdp', 'socp']
+__all__ = ['conelp', 'coneqp', 'lp', 'socp', 'sdp', 'qp', 'cp', 'cpl', 'gp']
